@@ -57,9 +57,24 @@
 #include <stdlib.h>
 #endif
 
-#include "locatedb.h"
+#if ENABLE_NLS
+# include <libintl.h>
+# define _(Text) gettext (Text)
+#else
+# define _(Text) Text
+#define textdomain(Domain)
+#define bindtextdomain(Package, Directory)
+#endif
+#ifdef gettext_noop
+# define N_(String) gettext_noop (String)
+#else
+# define N_(String) (String)
+#endif
 
-char *xmalloc ();
+#include "locatedb.h"
+#include <getline.h>
+
+char *xmalloc PARAMS((size_t));
 
 /* The name this program was run with.  */
 char *program_name;
@@ -71,8 +86,7 @@ static char bigrams[257] = {0};
 /* Return the offset of PATTERN in STRING, or -1 if not found. */
 
 static int
-strindex (string, pattern)
-     char *string, *pattern;
+strindex (char *string, char *pattern)
 {
   register char *s;
 
@@ -92,8 +106,7 @@ strindex (string, pattern)
 /* Return the length of the longest common prefix of strings S1 and S2. */
 
 static int
-prefix_length (s1, s2)
-     char *s1, *s2;
+prefix_length (char *s1, char *s2)
 {
   register char *start;
 
@@ -102,10 +115,8 @@ prefix_length (s1, s2)
   return s1 - start;
 }
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   char *path;			/* The current input entry.  */
   char *oldpath;		/* The previous input entry.  */
@@ -122,7 +133,7 @@ main (argc, argv)
 
   if (argc != 2)
     {
-      fprintf (stderr, "Usage: %s most_common_bigrams < list > coded_list\n",
+      fprintf (stderr, _("Usage: %s most_common_bigrams < list > coded_list\n"),
 	       argv[0]);
       exit (2);
     }
@@ -135,7 +146,7 @@ main (argc, argv)
       exit (1);
     }
 
-  pathsize = oldpathsize = 1026; /* Increased as necessary by getstr.  */
+  pathsize = oldpathsize = 1026; /* Increased as necessary by getline.  */
   path = xmalloc (pathsize);
   oldpath = xmalloc (oldpathsize);
 
@@ -150,7 +161,7 @@ main (argc, argv)
   fwrite (bigrams, 1, 256, stdout);
   fclose (fp);
 
-  while ((line_len = getstr (&path, &pathsize, stdin, '\n', 0)) > 0)
+  while ((line_len = getline (&path, &pathsize, stdin)) > 0)
     {
       char *pp;
 
